@@ -5,7 +5,7 @@
  *           (C) 2009 Adrian Perez
  * Authors:
  *   Adrian Perez <aperez@igalia.com>
- *   Mario Sanchez Prada <msanchez@igalia.com>
+ *   Mario Sanchez Prada <msanchez@gnome.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 3 of the GNU General Public
@@ -42,11 +42,15 @@ struct _FrogrAccountPrivate
   gchar *fullname;
   gchar *version;
   gboolean is_active;
+  gboolean has_extra_info;
 
   /* Following properties won't be persistent */
   gulong remaining_bandwidth;
   gulong max_bandwidth;
-  gulong max_filesize;
+  gulong max_picture_filesize;
+  gulong max_video_filesize;
+  guint remaining_videos;
+  guint current_videos;
   gboolean is_pro;
 };
 
@@ -62,9 +66,13 @@ enum {
   PROP_FULLNAME,
   PROP_VERSION,
   PROP_IS_ACTIVE,
+  PROP_HAS_EXTRA_INFO,
   PROP_REMAINING_BANDWIDTH,
   PROP_MAX_BANDWIDTH,
-  PROP_MAX_FILESIZE,
+  PROP_MAX_PICTURE_FILESIZE,
+  PROP_MAX_VIDEO_FILESIZE,
+  PROP_REMAINING_VIDEOS,
+  PROP_CURRENT_VIDEOS,
   PROP_IS_PRO
 };
 
@@ -113,6 +121,10 @@ _frogr_account_set_property (GObject      *object,
       frogr_account_set_is_active (self, g_value_get_boolean (value));
       break;
 
+    case PROP_HAS_EXTRA_INFO:
+      frogr_account_set_has_extra_info (self, g_value_get_boolean (value));
+      break;
+
     case PROP_REMAINING_BANDWIDTH:
       frogr_account_set_remaining_bandwidth (self, g_value_get_ulong (value));
       break;
@@ -121,8 +133,20 @@ _frogr_account_set_property (GObject      *object,
       frogr_account_set_max_bandwidth (self, g_value_get_ulong (value));
       break;
 
-    case PROP_MAX_FILESIZE:
-      frogr_account_set_max_filesize (self, g_value_get_ulong (value));
+    case PROP_MAX_PICTURE_FILESIZE:
+      frogr_account_set_max_picture_filesize (self, g_value_get_ulong (value));
+      break;
+
+    case PROP_MAX_VIDEO_FILESIZE:
+      frogr_account_set_max_video_filesize (self, g_value_get_ulong (value));
+      break;
+
+    case PROP_REMAINING_VIDEOS:
+      frogr_account_set_remaining_videos (self, g_value_get_uint (value));
+      break;
+
+    case PROP_CURRENT_VIDEOS:
+      frogr_account_set_current_videos (self, g_value_get_uint (value));
       break;
 
     case PROP_IS_PRO:
@@ -176,6 +200,10 @@ _frogr_account_get_property (GObject    *object,
       g_value_set_boolean (value, priv->is_active);
       break;
 
+    case PROP_HAS_EXTRA_INFO:
+      g_value_set_boolean (value, priv->has_extra_info);
+      break;
+
     case PROP_REMAINING_BANDWIDTH:
       g_value_set_ulong (value, priv->remaining_bandwidth);
       break;
@@ -184,8 +212,20 @@ _frogr_account_get_property (GObject    *object,
       g_value_set_ulong (value, priv->max_bandwidth);
       break;
 
-    case PROP_MAX_FILESIZE:
-      g_value_set_ulong (value, priv->max_filesize);
+    case PROP_MAX_PICTURE_FILESIZE:
+      g_value_set_ulong (value, priv->max_picture_filesize);
+      break;
+
+    case PROP_MAX_VIDEO_FILESIZE:
+      g_value_set_ulong (value, priv->max_video_filesize);
+      break;
+
+    case PROP_REMAINING_VIDEOS:
+      g_value_set_uint (value, priv->remaining_videos);
+      break;
+
+    case PROP_CURRENT_VIDEOS:
+      g_value_set_uint (value, priv->current_videos);
       break;
 
     case PROP_IS_PRO:
@@ -281,6 +321,14 @@ frogr_account_class_init (FrogrAccountClass *klass)
                                 G_PARAM_READWRITE);
   g_object_class_install_property (obj_class, PROP_IS_ACTIVE, pspec);
 
+  pspec = g_param_spec_boolean ("has-extra-info",
+                                "has-extra-info",
+                                "Whether the account has been filled with "
+                                "extra information as retrieved from flickr",
+                                FALSE,
+                                G_PARAM_READWRITE);
+  g_object_class_install_property (obj_class, PROP_HAS_EXTRA_INFO, pspec);
+
   pspec = g_param_spec_ulong ("remaining-bandwidth",
                               "remaining-bandwidth",
                               "Remaining monthly bandwidth in KB",
@@ -295,12 +343,33 @@ frogr_account_class_init (FrogrAccountClass *klass)
                               G_PARAM_READWRITE);
   g_object_class_install_property (obj_class, PROP_MAX_BANDWIDTH, pspec);
 
-  pspec = g_param_spec_ulong ("max-filesize",
-                              "max-filesize",
-                              "Max allowed filesize in KB",
+  pspec = g_param_spec_ulong ("max-picture-filesize",
+                              "max-picture-filesize",
+                              "Max allowed filesize for pictures in KB",
                               0, G_MAXULONG, G_MAXULONG,
                               G_PARAM_READWRITE);
-  g_object_class_install_property (obj_class, PROP_MAX_FILESIZE, pspec);
+  g_object_class_install_property (obj_class, PROP_MAX_PICTURE_FILESIZE, pspec);
+
+  pspec = g_param_spec_ulong ("max-video-filesize",
+                              "max-video-filesize",
+                              "Max allowed filesize for videos in KB",
+                              0, G_MAXULONG, G_MAXULONG,
+                              G_PARAM_READWRITE);
+  g_object_class_install_property (obj_class, PROP_MAX_VIDEO_FILESIZE, pspec);
+
+  pspec = g_param_spec_uint ("remaining-videos",
+                             "remaining-videos",
+                             "Remaining number of videos for upload",
+                             0, G_MAXUINT, G_MAXUINT,
+                             G_PARAM_READWRITE);
+  g_object_class_install_property (obj_class, PROP_REMAINING_VIDEOS, pspec);
+
+  pspec = g_param_spec_uint ("current-videos",
+                             "current-videos",
+                             "Current number of videos uploaded",
+                             0, G_MAXUINT, G_MAXUINT,
+                             G_PARAM_READWRITE);
+  g_object_class_install_property (obj_class, PROP_CURRENT_VIDEOS, pspec);
 
   pspec = g_param_spec_boolean ("is-pro",
                                 "is-pro",
@@ -324,9 +393,13 @@ frogr_account_init (FrogrAccount *self)
   priv->fullname = NULL;
   priv->version = NULL;
   priv->is_active = FALSE;
+  priv->has_extra_info = FALSE;
   priv->remaining_bandwidth = G_MAXULONG;
   priv->max_bandwidth = G_MAXULONG;
-  priv->max_filesize = G_MAXULONG;
+  priv->max_picture_filesize = G_MAXULONG;
+  priv->max_video_filesize = G_MAXULONG;
+  priv->remaining_videos = G_MAXUINT;
+  priv->current_videos = G_MAXUINT;
   priv->is_pro = FALSE;
 }
 
@@ -515,6 +588,28 @@ frogr_account_set_is_active (FrogrAccount *self, gboolean is_active)
   priv->is_active = is_active;
 }
 
+gboolean
+frogr_account_has_extra_info (FrogrAccount *self)
+{
+  FrogrAccountPrivate *priv = NULL;
+
+  g_return_val_if_fail (FROGR_IS_ACCOUNT (self), FALSE);
+
+  priv = FROGR_ACCOUNT_GET_PRIVATE (self);
+  return priv->has_extra_info;
+}
+
+void
+frogr_account_set_has_extra_info (FrogrAccount *self, gboolean has_extra_info)
+{
+  FrogrAccountPrivate *priv = NULL;
+
+  g_return_if_fail (FROGR_IS_ACCOUNT (self));
+
+  priv = FROGR_ACCOUNT_GET_PRIVATE (self);
+  priv->has_extra_info = has_extra_info;
+}
+
 gulong
 frogr_account_get_remaining_bandwidth (FrogrAccount *self)
 {
@@ -559,27 +654,93 @@ frogr_account_set_max_bandwidth (FrogrAccount *self, gulong max_bandwidth)
   priv->max_bandwidth = max_bandwidth;
 }
 
-gulong frogr_account_get_max_filesize (FrogrAccount *self)
+gulong frogr_account_get_max_picture_filesize (FrogrAccount *self)
 {
   FrogrAccountPrivate *priv = NULL;
 
   g_return_val_if_fail (FROGR_IS_ACCOUNT (self), G_MAXULONG);
 
   priv = FROGR_ACCOUNT_GET_PRIVATE (self);
-  return priv->max_filesize;
+  return priv->max_picture_filesize;
 }
 
-void frogr_account_set_max_filesize (FrogrAccount *self,
-                                     gulong max_filesize)
+void frogr_account_set_max_picture_filesize (FrogrAccount *self,
+                                           gulong max_filesize)
 {
   FrogrAccountPrivate *priv = NULL;
 
   g_return_if_fail (FROGR_IS_ACCOUNT (self));
 
   priv = FROGR_ACCOUNT_GET_PRIVATE (self);
-  priv->max_filesize = max_filesize;
+  priv->max_picture_filesize = max_filesize;
 }
 
+gulong
+frogr_account_get_max_video_filesize (FrogrAccount *self)
+{
+  FrogrAccountPrivate *priv = NULL;
+
+  g_return_val_if_fail (FROGR_IS_ACCOUNT (self), G_MAXULONG);
+
+  priv = FROGR_ACCOUNT_GET_PRIVATE (self);
+  return priv->max_video_filesize;
+
+}
+
+void
+frogr_account_set_max_video_filesize (FrogrAccount *self, gulong max_filesize)
+{
+  FrogrAccountPrivate *priv = NULL;
+
+  g_return_if_fail (FROGR_IS_ACCOUNT (self));
+
+  priv = FROGR_ACCOUNT_GET_PRIVATE (self);
+  priv->max_video_filesize = max_filesize;
+}
+
+gulong
+frogr_account_get_remaining_videos (FrogrAccount *self)
+{
+  FrogrAccountPrivate *priv = NULL;
+
+  g_return_val_if_fail (FROGR_IS_ACCOUNT (self), G_MAXUINT);
+
+  priv = FROGR_ACCOUNT_GET_PRIVATE (self);
+  return priv->remaining_videos;
+}
+
+void
+frogr_account_set_remaining_videos (FrogrAccount *self, guint remaining_videos)
+{
+  FrogrAccountPrivate *priv = NULL;
+
+  g_return_if_fail (FROGR_IS_ACCOUNT (self));
+
+  priv = FROGR_ACCOUNT_GET_PRIVATE (self);
+  priv->remaining_videos = remaining_videos;
+}
+
+guint
+frogr_account_get_current_videos (FrogrAccount *self)
+{
+  FrogrAccountPrivate *priv = NULL;
+
+  g_return_val_if_fail (FROGR_IS_ACCOUNT (self), G_MAXUINT);
+
+  priv = FROGR_ACCOUNT_GET_PRIVATE (self);
+  return priv->current_videos;
+}
+
+void
+frogr_account_set_current_videos (FrogrAccount *self, guint current_videos)
+{
+  FrogrAccountPrivate *priv = NULL;
+
+  g_return_if_fail (FROGR_IS_ACCOUNT (self));
+
+  priv = FROGR_ACCOUNT_GET_PRIVATE (self);
+  priv->current_videos = current_videos;
+}
 
 gboolean
 frogr_account_is_pro (FrogrAccount *self)
