@@ -1,7 +1,7 @@
 /*
  * frogr-add-tags-dialog.c -- Frogr 'add tags' dialog
  *
- * Copyright (C) 2009-2012 Mario Sanchez Prada
+ * Copyright (C) 2009-2014 Mario Sanchez Prada
  * Authors: Mario Sanchez Prada <msanchez@gnome.org>
  *
  * This program is free software; you can redistribute it and/or
@@ -195,9 +195,9 @@ frogr_add_tags_dialog_init (FrogrAddTagsDialog *self)
 
   /* Create widgets */
   gtk_dialog_add_buttons (GTK_DIALOG (self),
-                          GTK_STOCK_CANCEL,
+                          _("_Cancel"),
                           GTK_RESPONSE_CANCEL,
-                          GTK_STOCK_ADD,
+                          _("_Add"),
                           GTK_RESPONSE_ACCEPT,
                           NULL);
   gtk_container_set_border_width (GTK_CONTAINER (self), 6);
