@@ -1,7 +1,7 @@
 /*
  * frogr-auth-dialog.c -- Authorization dialog
  *
- * Copyright (C) 2009-2012 Mario Sanchez Prada
+ * Copyright (C) 2009-2014 Mario Sanchez Prada
  * Authors: Mario Sanchez Prada <msanchez@gnome.org>
  *
  * This program is free software; you can redistribute it and/or
@@ -142,7 +142,7 @@ _ask_for_auth_confirmation (GtkWindow *parent)
   dialog = gtk_dialog_new_with_buttons (title,
                                         parent,
                                         GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-                                        GTK_STOCK_CLOSE,
+                                        _("_Close"),
                                         GTK_RESPONSE_CLOSE,
                                         NULL);
   g_free (title);
