@@ -1,8 +1,8 @@
 /*
  * frogr-details-dialog.h -- Picture details dialog
  *
- * Copyright (C) 2009-2011 Mario Sanchez Prada
- * Authors: Mario Sanchez Prada <msanchez@igalia.com>
+ * Copyright (C) 2009-2012 Mario Sanchez Prada
+ * Authors: Mario Sanchez Prada <msanchez@gnome.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 3 of the GNU General Public
@@ -47,7 +47,9 @@ struct _FrogrDetailsDialog
 
 GType frogr_details_dialog_get_type (void) G_GNUC_CONST;
 
-void frogr_details_dialog_show (GtkWindow *parent, GSList *fpictures, GSList *tags);
+void frogr_details_dialog_show (GtkWindow *parent,
+                                const GSList *pictures,
+                                const GSList *tags);
 
 G_END_DECLS  /* FROGR_DETAILS_DIALOG_H */
 
