@@ -1,7 +1,7 @@
 /*
  * frogr-main-view.c -- Main view for the application
  *
- * Copyright (C) 2009-2012 Mario Sanchez Prada
+ * Copyright (C) 2009-2014 Mario Sanchez Prada
  *           (C) 2011 Joaquim Rocha
  * Authors:
  *   Joaquim Rocha <jrocha@igalia.com>
@@ -46,6 +46,7 @@
 #define UI_MAIN_VIEW_FILE "/gtkbuilder/frogr-main-view.xml"
 #define UI_APP_MENU_FILE "/gtkbuilder/frogr-app-menu.xml"
 #define UI_MENU_BAR_FILE "/gtkbuilder/frogr-menu-bar.xml"
+#define UI_CONTEXT_MENU_FILE "/gtkbuilder/frogr-context-menu.xml"
 
 /* Action names for menu items */
 #define ACTION_AUTHORIZE "authorize"
@@ -74,6 +75,12 @@
 #define ACTION_SORT_IN_REVERSE_ORDER "sort-in-reverse-order"
 #define ACTION_ENABLE_TOOLTIPS "enable-tooltips"
 
+/* This should map to Control in X11 and to Command in OS X */
+#ifdef PLATFORM_MAC
+#define GDK_PRIMARY_MASK GDK_MOD2_MASK
+#else
+#define GDK_PRIMARY_MASK GDK_CONTROL_MASK
+#endif
 
 #define FROGR_MAIN_VIEW_GET_PRIVATE(object)             \
   (G_TYPE_INSTANCE_GET_PRIVATE ((object),               \
@@ -112,15 +119,11 @@ typedef struct _FrogrMainViewPrivate {
   gchar* state_description;
 
   GtkTreeModel *tree_model;
+  GtkTreePath *reference_picture;
   guint sb_context_id;
 
   GtkBuilder *builder;
   GMenuModel *app_menu;
-
-  /* For the toolbar and the contextual menu */
-  GtkActionGroup *file_gtkactions;
-  GtkActionGroup *pictures_gtkactions;
-  GtkActionGroup *selection_gtkactions;
 } FrogrMainViewPrivate;
 
 
@@ -134,6 +137,7 @@ enum {
 /* Prototypes */
 
 static void _initialize_ui (FrogrMainView *self);
+static GtkToolItem *_create_toolbar_item (const gchar *action_name, const gchar *icon_name, const gchar *label, const gchar *tooltip_text);
 static gboolean _maybe_show_auth_dialog_on_idle (FrogrMainView *self);
 
 static void _update_project_path (FrogrMainView *self, const gchar *path);
@@ -145,9 +149,6 @@ static void _on_radio_menu_item_changed (GSimpleAction *action, GVariant *parame
 static void _on_toggle_menu_item_activated (GSimpleAction *action, GVariant *parameter, gpointer data);
 static void _on_toggle_menu_item_changed (GSimpleAction *action, GVariant *parameter, gpointer data);
 
-/* This needs to be non-static because of GtkBuilder UI definition file */
-void _on_gtk_action_activated (GtkAction *action, gpointer data);
-
 static void _quit_application (FrogrMainView *self);
 
 #ifdef PLATFORM_MAC
@@ -167,6 +168,14 @@ static void _on_icon_view_drag_data_received (GtkWidget *widget,
 gboolean _on_icon_view_key_press_event (GtkWidget *widget,
                                         GdkEventKey *event,
                                         gpointer data);
+
+static void _clear_reference_picture (FrogrMainView *self);
+static void _update_reference_picture (FrogrMainView *self,
+                                       GtkTreePath *new_path);
+static void _handle_selection_for_button_event_and_path (FrogrMainView *self,
+                                                         GdkEventButton *event,
+                                                         GtkTreePath *path);
+
 gboolean _on_icon_view_button_press_event (GtkWidget *widget,
                                            GdkEventButton *event,
                                            gpointer data);
@@ -184,6 +193,7 @@ static gboolean _on_icon_view_query_tooltip (GtkWidget *icon_view,
 static void _on_icon_view_selection_changed (GtkWidget *icon_view,
                                              gpointer data);
 
+static void _deselect_all_pictures (FrogrMainView *self);
 static GSList *_get_selected_pictures (FrogrMainView *self);
 static gint _n_pictures (FrogrMainView *self);
 static void _open_pictures_in_external_viewer (FrogrMainView *self);
@@ -310,6 +320,8 @@ _initialize_ui (FrogrMainView *self)
   GtkWidget *progress_bar;
   GtkWidget *progress_label;
   GtkWidget *toolbar;
+  GtkToolItem *toolbar_item;
+  GMenuModel *ctxt_menu_model;
   const gchar *icons_path = NULL;
   gchar *full_path = NULL;
   GList *icons = NULL;
@@ -384,7 +396,22 @@ _initialize_ui (FrogrMainView *self)
                                G_MENU_MODEL (gtk_builder_get_object (builder, "menu-bar")));
   gtk_application_window_set_show_menubar (GTK_APPLICATION_WINDOW (self), TRUE);
 
+  /* Toolbar */
   toolbar = GTK_WIDGET (gtk_builder_get_object (builder, "toolbar"));
+  toolbar_item = _create_toolbar_item ("win.open-project", "gtk-open", _("Open"), _("Open Existing Project"));
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), toolbar_item, 0);
+  toolbar_item = _create_toolbar_item ("win.save-project", "gtk-save", _("Save"), _("Save Current Project"));
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), toolbar_item, 1);
+  toolbar_item = gtk_separator_tool_item_new ();
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), toolbar_item, 2);
+  toolbar_item = _create_toolbar_item ("win.add-pictures", "gtk-add", _("Add"), _("Add Elements"));
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), toolbar_item, 3);
+  toolbar_item = _create_toolbar_item ("win.remove-pictures", "gtk-remove", _("Remove"), _("Remove Elements"));
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), toolbar_item, 4);
+  toolbar_item = gtk_separator_tool_item_new ();
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), toolbar_item, 5);
+  toolbar_item = _create_toolbar_item ("win.upload-all", "gtk-go-up", _("Upload"), _("Upload All"));
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), toolbar_item, 6);
   gtk_style_context_add_class (gtk_widget_get_style_context (toolbar),
                                GTK_STYLE_CLASS_PRIMARY_TOOLBAR);
 
@@ -394,11 +421,6 @@ _initialize_ui (FrogrMainView *self)
   status_bar = GTK_WIDGET (gtk_builder_get_object (builder, "status_bar"));
   priv->status_bar = status_bar;
 
-  /* Get action groups from GtkBuilder for the toolbar and the context menu */
-  priv->file_gtkactions = GTK_ACTION_GROUP (gtk_builder_get_object (builder, "file-actions"));
-  priv->pictures_gtkactions = GTK_ACTION_GROUP (gtk_builder_get_object (builder, "pictures-actions"));
-  priv->selection_gtkactions = GTK_ACTION_GROUP (gtk_builder_get_object (builder, "selection-actions"));
-
   /* Init main model's state description */
   _update_state_description (self);
 
@@ -430,8 +452,12 @@ _initialize_ui (FrogrMainView *self)
   _populate_accounts_submenu (self);
 
   /* Create contextual menus for right-clicks */
-  priv->pictures_ctxt_menu =
-      GTK_WIDGET (gtk_builder_get_object (builder, "ctxt_menu"));
+  full_path = g_strdup_printf ("%s/" UI_CONTEXT_MENU_FILE, frogr_util_get_app_data_dir ());
+  gtk_builder_add_from_file (builder, full_path, NULL);
+  g_free (full_path);
+  ctxt_menu_model = G_MENU_MODEL (gtk_builder_get_object (builder, "context-menu"));
+  priv->pictures_ctxt_menu = gtk_menu_new_from_model (ctxt_menu_model);
+  gtk_menu_attach_to_widget (GTK_MENU (priv->pictures_ctxt_menu), GTK_WIDGET (self), NULL);
 
   /* Initialize drag'n'drop support */
   _initialize_drag_n_drop (self);
@@ -440,7 +466,7 @@ _initialize_ui (FrogrMainView *self)
   progress_dialog = gtk_dialog_new_with_buttons (NULL,
                                                  GTK_WINDOW (self),
                                                  GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-                                                 GTK_STOCK_CANCEL,
+                                                 _("_Cancel"),
                                                  GTK_RESPONSE_CANCEL,
                                                  NULL);
   gtk_window_set_title (GTK_WINDOW (progress_dialog), APP_SHORTNAME);
@@ -472,7 +498,7 @@ _initialize_ui (FrogrMainView *self)
   priv->tree_model = GTK_TREE_MODEL (gtk_list_store_new (3,
                                                          G_TYPE_STRING,
                                                          GDK_TYPE_PIXBUF,
-                                                         G_TYPE_POINTER));
+                                                         G_TYPE_OBJECT));
   gtk_icon_view_set_model (GTK_ICON_VIEW (icon_view), priv->tree_model);
   gtk_icon_view_set_pixbuf_column (GTK_ICON_VIEW (icon_view), PIXBUF_COL);
   gtk_icon_view_set_selection_mode (GTK_ICON_VIEW (icon_view),
@@ -532,6 +558,20 @@ _initialize_ui (FrogrMainView *self)
   gtk_widget_show_all (GTK_WIDGET (self));
 }
 
+static GtkToolItem *
+_create_toolbar_item (const gchar *action_name, const gchar *icon_name, const gchar *label, const gchar *tooltip_text)
+{
+  GtkWidget *widget = NULL;
+  GtkToolItem *item = NULL;
+
+  widget = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_SMALL_TOOLBAR);
+  item = gtk_tool_button_new (widget, label);
+  gtk_tool_item_set_tooltip_text (item, tooltip_text);
+  gtk_actionable_set_action_name (GTK_ACTIONABLE (item), action_name);
+
+  return item;
+}
+
 static gboolean
 _maybe_show_auth_dialog_on_idle (FrogrMainView *self)
 {
@@ -666,7 +706,7 @@ _on_menu_item_activated (GSimpleAction *action, GVariant *parameter, gpointer da
   else if (!g_strcmp0 (action_name, ACTION_ABOUT))
     frogr_controller_show_about_dialog (priv->controller);
   else if (!g_strcmp0 (action_name, ACTION_HELP))
-    frogr_util_open_uri ("ghelp:frogr");
+    frogr_util_open_uri ("help:frogr");
   else if (!g_strcmp0 (action_name, ACTION_QUIT))
     _quit_application (self);
   else
@@ -767,41 +807,6 @@ _quit_application (FrogrMainView *self)
   g_application_quit (G_APPLICATION (gtk_app));
 }
 
-void
-_on_gtk_action_activated (GtkAction *action, gpointer data)
-{
-  FrogrMainView *mainview = FROGR_MAIN_VIEW (data);
-  const gchar *action_name = NULL;
-
-  action_name = gtk_action_get_name (action);
-  if (!g_strcmp0 (action_name, ACTION_UPLOAD_ALL))
-    _upload_pictures (mainview);
-  else if (!g_strcmp0 (action_name, ACTION_EDIT_DETAILS))
-    _edit_selected_pictures (mainview);
-  else if (!g_strcmp0 (action_name, ACTION_ADD_TAGS))
-    _add_tags_to_pictures (mainview);
-  else if (!g_strcmp0 (action_name, ACTION_ADD_TO_GROUP))
-    _add_pictures_to_group (mainview);
-  else if (!g_strcmp0 (action_name, ACTION_ADD_TO_SET))
-    _add_pictures_to_existing_set (mainview);
-  else if (!g_strcmp0 (action_name, ACTION_ADD_TO_NEW_SET))
-    _add_pictures_to_new_set (mainview);
-  else if (!g_strcmp0 (action_name, ACTION_OPEN_IN_EXTERNAL_VIEWER))
-    _open_pictures_in_external_viewer (mainview);
-  else if (!g_strcmp0 (action_name, ACTION_ADD_PICTURES))
-    _load_pictures_dialog (mainview);
-  else if (!g_strcmp0 (action_name, ACTION_REMOVE_PICTURES))
-    _remove_selected_pictures (mainview);
-  else if (!g_strcmp0 (action_name, ACTION_OPEN_PROJECT))
-    _open_project_dialog (mainview);
-  else if (!g_strcmp0 (action_name, ACTION_SAVE_PROJECT))
-    _save_current_project (mainview);
-  else if (!g_strcmp0 (action_name, ACTION_SAVE_PROJECT_AS))
-    _save_project_as_dialog (mainview);
-  else
-    g_assert_not_reached ();
-}
-
 #ifdef PLATFORM_MAC
 static void
 _tweak_app_menu_for_mac (FrogrMainView *self)
@@ -909,7 +914,6 @@ _on_icon_view_drag_data_received (GtkWidget *widget,
   FrogrMainViewPrivate *priv = NULL;
   GdkAtom target;
   GSList *fileuris_list = NULL;
-  const guchar *files_string = NULL;
   gchar **fileuris_array = NULL;
   gint i;
 
@@ -921,14 +925,10 @@ _on_icon_view_drag_data_received (GtkWidget *widget,
     return;
 
   target = gtk_selection_data_get_target (selection_data);
-
   if (!gtk_targets_include_uri (&target, 1))
     return;
 
-  /* Get GSList with the list of files */
-  files_string = gtk_selection_data_get_data (selection_data);
-
-  fileuris_array = g_strsplit ((const gchar*)files_string, "\r\n", -1);
+  fileuris_array = gtk_selection_data_get_uris (selection_data);;
   for (i = 0;  fileuris_array[i]; i++)
     {
       if (fileuris_array[i] && fileuris_array[i][0] != '\0')
@@ -978,6 +978,132 @@ _on_icon_view_key_press_event (GtkWidget *widget,
   return FALSE;
 }
 
+static void
+_clear_reference_picture (FrogrMainView *self)
+{
+  FrogrMainViewPrivate *priv = FROGR_MAIN_VIEW_GET_PRIVATE (self);
+  if (priv->reference_picture)
+    {
+      gtk_tree_path_free (priv->reference_picture);
+      priv->reference_picture = NULL;
+    }
+}
+
+static void
+_update_reference_picture (FrogrMainView *self, GtkTreePath *new_path)
+{
+  FrogrMainViewPrivate *priv = FROGR_MAIN_VIEW_GET_PRIVATE (self);
+
+  _clear_reference_picture (self);
+  if (new_path)
+    priv->reference_picture = gtk_tree_path_copy (new_path);
+}
+
+static void
+_handle_selection_for_button_event_and_path (FrogrMainView *mainview,
+                                             GdkEventButton *event,
+                                             GtkTreePath *path)
+{
+  FrogrMainViewPrivate *priv = FROGR_MAIN_VIEW_GET_PRIVATE (mainview);
+  gboolean using_primary_key = event->state & GDK_PRIMARY_MASK;
+  gboolean using_shift_key = event->state & GDK_SHIFT_MASK;
+  gboolean is_single_click = event->type == GDK_BUTTON_PRESS;
+  gboolean is_primary_btn = event->button == 1;
+  gboolean path_selected = gtk_icon_view_path_is_selected (GTK_ICON_VIEW (priv->icon_view), path);
+
+  /* Clicking with the secondary button in a selected element means
+     we just want to open the contextual menu, so do nothing */
+  if (path_selected && !is_primary_btn)
+    return;
+
+  /* Handle simple cases (no modifiers) */
+  if (!using_primary_key && !using_shift_key)
+    {
+      gint n_selected_pictures = priv->n_selected_pictures;
+
+      _deselect_all_pictures (mainview);
+
+      /* We will just select the pointed element if it's not selected yet
+         or if it is, but it belongs to a multiple selection previously done */
+      if (!path_selected || n_selected_pictures > 1)
+        {
+          gtk_icon_view_select_path (GTK_ICON_VIEW (priv->icon_view), path);
+          _update_reference_picture (mainview, path);
+        }
+
+      /* Already handled */
+      return;
+    }
+
+  /* Handle special cases with Control/Command */
+  if (using_primary_key)
+    {
+      /* Ignore double and triple clicks with Control/Command (treat them as separate clicks) */
+      if (!is_single_click)
+        return;
+
+      /* When combined with Shift and clicking in a not previously selected
+         item, we will just extend the selection through the 'Shift' code path. */
+      if (!using_shift_key || path_selected)
+        {
+          if (!using_shift_key)
+            _clear_reference_picture (mainview);
+
+          if (path_selected)
+            gtk_icon_view_unselect_path (GTK_ICON_VIEW (priv->icon_view), path);
+          else
+            {
+              gtk_icon_view_select_path (GTK_ICON_VIEW (priv->icon_view), path);
+              _update_reference_picture (mainview, path);
+            }
+
+          /* Already handled */
+          return;
+        }
+    }
+
+  /* Finally, handle special cases with Shift */
+  if (priv->reference_picture)
+    {
+      GtkTreeIter start_iter;
+      if (gtk_tree_model_get_iter (priv->tree_model, &start_iter, priv->reference_picture))
+        {
+          GtkTreePath *start_path = priv->reference_picture;
+          GtkTreePath *end_path = path;
+          GtkTreePath *current_path = NULL;
+
+          /* Swap paths if needed */
+          if (gtk_tree_path_compare (start_path, end_path) > 0)
+            {
+              GtkTreePath *tmp_path = start_path;
+              start_path = end_path;
+              end_path = tmp_path;
+            }
+
+          /* If not using Control/Command (otherwise we would keep the current selection)
+             we first unselect everything and then start selecting, but keeping
+             the reference icon anyway, since we will just extend from there */
+          if (!using_primary_key)
+            gtk_icon_view_unselect_all (GTK_ICON_VIEW (priv->icon_view));
+
+          current_path = gtk_tree_path_copy (start_path);
+          while (gtk_tree_path_compare (current_path, end_path) <= 0)
+            {
+              gtk_icon_view_select_path (GTK_ICON_VIEW (priv->icon_view), current_path);
+              gtk_tree_path_next(current_path);
+            }
+          gtk_tree_path_free (current_path);
+        }
+
+      /* Already handled */
+      return;
+    }
+
+  /* If nothing was previously selected when using Shift, treat it as a normal click */
+  gtk_icon_view_select_path (GTK_ICON_VIEW (priv->icon_view), path);
+  _update_reference_picture (mainview, path);
+}
+
 gboolean
 _on_icon_view_button_press_event (GtkWidget *widget,
                                   GdkEventButton *event,
@@ -985,65 +1111,60 @@ _on_icon_view_button_press_event (GtkWidget *widget,
 {
   FrogrMainView *mainview = FROGR_MAIN_VIEW (data);
   FrogrMainViewPrivate *priv = FROGR_MAIN_VIEW_GET_PRIVATE (data);
-  GtkTreePath *path;
+  gboolean using_primary_key = event->state & GDK_PRIMARY_MASK;
+  gboolean using_shift_key = event->state & GDK_SHIFT_MASK;
+  GtkTreePath *new_path = NULL;
+
+  /* Grab the focus, as we will be handling events manually */
+  gtk_widget_grab_focus(widget);
 
   /* Actions are only allowed in IDLE state */
   if (frogr_controller_get_state (priv->controller) != FROGR_STATE_IDLE)
     return TRUE;
 
+  /* Only left and right clicks are supported */
+  if (event->button != 1 && event->button != 3)
+    return TRUE;
+
   /* Do nothing if there's no picture loaded yet */
   if (!_n_pictures (mainview))
     return TRUE;
 
   /* Check if we clicked on top of an item */
   if (gtk_icon_view_get_item_at_pos (GTK_ICON_VIEW (priv->icon_view),
-                                     event->x,
-                                     event->y,
-                                     &path,
-                                     NULL))
+                                     event->x, event->y, &new_path, NULL))
     {
-      gboolean path_selected =
-        gtk_icon_view_path_is_selected (GTK_ICON_VIEW (priv->icon_view), path);
+      gboolean is_primary_btn = event->button == 1;
+      gboolean is_single_click = event->type == GDK_BUTTON_PRESS;
+      gboolean is_double_click = event->type == GDK_2BUTTON_PRESS;
 
-      /* Check whether it's needed to keep this item as the only selection */
-      if (((event->button == 1) && path_selected)
-          || ((event->button == 3) && !path_selected))
-        {
-          if (!(event->state & GDK_SHIFT_MASK)
-              && !(event->state & GDK_CONTROL_MASK))
-            {
-              /* Deselect all items if not pressing Ctrl or shift */
-              gtk_icon_view_unselect_all (GTK_ICON_VIEW (priv->icon_view));
-            }
-
-          /* Now select the item */
-          gtk_icon_view_select_path (GTK_ICON_VIEW (priv->icon_view), path);
-
-        }
+      /* Decide whether we need to change the selection and how */
+      _handle_selection_for_button_event_and_path (mainview, event, new_path);
 
       /* Perform the right action: edit picture or show ctxt menu */
-      if ((event->button == 1)                   /* left button */
-          && (event->type == GDK_2BUTTON_PRESS ) /* doubleclick */
-          && !(event->state & GDK_SHIFT_MASK)    /*  not shift  */
-          && !(event->state & GDK_CONTROL_MASK)) /*  not Ctrl */
+      if (is_primary_btn && is_double_click && !using_primary_key)
         {
           /* edit selected item */
           _edit_selected_pictures (mainview);
         }
-      else if ((event->button == 3)                  /* right button */
-               && (event->type == GDK_BUTTON_PRESS)) /* single click */
+      else if (!is_primary_btn && is_single_click)
         {
           /* Show contextual menu */
           gtk_menu_popup (GTK_MENU (priv->pictures_ctxt_menu),
-                          NULL, NULL, NULL, NULL,
-                          event->button,
-                          gtk_get_current_event_time ());
+              NULL, NULL, NULL, NULL,
+              event->button,
+              gtk_get_current_event_time ());
         }
 
       /* Free */
-      gtk_tree_path_free (path);
+      gtk_tree_path_free (new_path);
+      return TRUE;
     }
 
+  /* Make sure we reset everything if simply clicking outside of any picture*/
+  if (!using_primary_key && !using_shift_key)
+    _deselect_all_pictures (mainview);
+
   return FALSE;
 }
 
@@ -1130,6 +1251,7 @@ _on_icon_view_query_tooltip (GtkWidget *icon_view,
 
       /* Free memory */
       gtk_tree_path_free (path);
+      g_object_unref (picture);
       g_free (tooltip_str);
       g_free (filesize);
       g_free (filesize_str);
@@ -1165,6 +1287,15 @@ _on_icon_view_selection_changed (GtkWidget *icon_view, gpointer data)
   _update_sensitiveness (self);
 }
 
+static void
+_deselect_all_pictures (FrogrMainView *self)
+{
+  FrogrMainViewPrivate *priv = FROGR_MAIN_VIEW_GET_PRIVATE (self);
+
+  gtk_icon_view_unselect_all (GTK_ICON_VIEW (priv->icon_view));
+  _clear_reference_picture (self);
+}
+
 static GSList *
 _get_selected_pictures (FrogrMainView *self)
 {
@@ -1191,7 +1322,7 @@ _get_selected_pictures (FrogrMainView *self)
                           -1);
 
       /* Add the picture to the list */
-      pictures = g_slist_prepend (pictures, g_object_ref (picture));
+      pictures = g_slist_prepend (pictures, picture);
       gtk_tree_path_free (path);
     }
 
@@ -1244,8 +1375,8 @@ _open_project_dialog (FrogrMainView *self)
   dialog = gtk_file_chooser_dialog_new (_("Select File"),
                                         GTK_WINDOW (self),
                                         GTK_FILE_CHOOSER_ACTION_SAVE,
-                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                                        GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+                                        _("_Cancel"), GTK_RESPONSE_CANCEL,
+                                        _("_Open"), GTK_RESPONSE_ACCEPT,
                                         NULL);
 
   gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (dialog), FALSE);
@@ -1321,8 +1452,8 @@ _save_project_as_dialog (FrogrMainView *self)
   dialog = gtk_file_chooser_dialog_new (_("Select Destination"),
                                         GTK_WINDOW (self),
                                         GTK_FILE_CHOOSER_ACTION_SAVE,
-                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                                        GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+                                        _("_Cancel"), GTK_RESPONSE_CANCEL,
+                                        _("_Save"), GTK_RESPONSE_ACCEPT,
                                         NULL);
 
   gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), _("Untitled Project.frogr"));
@@ -1361,7 +1492,9 @@ _load_pictures_dialog (FrogrMainView *self)
   GtkWidget *dialog;
   GtkFileFilter *all_filter;
   GtkFileFilter *image_filter;
+#ifdef HAVE_GSTREAMER
   GtkFileFilter *video_filter;
+#endif
   gint i;
 
   const gchar * const *supported_mimetypes;
@@ -1369,22 +1502,26 @@ _load_pictures_dialog (FrogrMainView *self)
   dialog = gtk_file_chooser_dialog_new (_("Select a Picture"),
                                         GTK_WINDOW (self),
                                         GTK_FILE_CHOOSER_ACTION_OPEN,
-                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                                        GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+                                        _("_Cancel"), GTK_RESPONSE_CANCEL,
+                                        _("_Open"), GTK_RESPONSE_ACCEPT,
                                         NULL);
 
   /* Set images filter */
   all_filter = gtk_file_filter_new ();
   image_filter = gtk_file_filter_new ();
+#ifdef HAVE_GSTREAMER
   video_filter = gtk_file_filter_new ();
+#endif
 
   supported_mimetypes = frogr_util_get_supported_mimetypes ();
   for (i = 0; supported_mimetypes[i]; i++)
     {
       if (g_str_has_prefix (supported_mimetypes[i], "image"))
         gtk_file_filter_add_mime_type (image_filter, supported_mimetypes[i]);
+#ifdef HAVE_GSTREAMER
       else
         gtk_file_filter_add_mime_type (video_filter, supported_mimetypes[i]);
+#endif
 
       gtk_file_filter_add_mime_type (all_filter, supported_mimetypes[i]);
     }
@@ -1395,8 +1532,10 @@ _load_pictures_dialog (FrogrMainView *self)
   gtk_file_filter_set_name (image_filter, _("Image Files"));
   gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), image_filter);
 
+#ifdef HAVE_GSTREAMER
   gtk_file_filter_set_name (video_filter, _("Video Files"));
   gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), video_filter);
+#endif
 
   gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (dialog), TRUE);
   gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (dialog), FALSE);
@@ -1513,6 +1652,7 @@ _open_pictures_in_external_viewer (FrogrMainView *self)
 
   pictures = _get_selected_pictures (self);
   frogr_util_open_pictures_in_viewer (pictures);
+  g_slist_foreach (pictures, (GFunc)g_object_unref, NULL);
   g_slist_free (pictures);
 }
 
@@ -1538,6 +1678,7 @@ _remove_selected_pictures (FrogrMainView *self)
   _update_ui (self);
 
   /* Free */
+  g_slist_foreach (selected_pictures, (GFunc)g_object_unref, NULL);
   g_slist_free (selected_pictures);
 }
 
@@ -1556,7 +1697,7 @@ _upload_pictures (FrogrMainView *self)
   if (!_pictures_loaded_required_check (self))
     return;
 
-  gtk_icon_view_unselect_all (GTK_ICON_VIEW (priv->icon_view));
+  _deselect_all_pictures (self);
   frogr_controller_upload_pictures (priv->controller, priv->sorted_pictures);
 }
 
@@ -1815,6 +1956,7 @@ _model_picture_removed (FrogrController *controller,
   if (gtk_tree_model_get_iter_first (tree_model, &iter))
     {
       /* Look for the picture and remove it */
+      gboolean found = FALSE;
       do
         {
           FrogrPicture *picture_from_ui;
@@ -1834,10 +1976,11 @@ _model_picture_removed (FrogrController *controller,
               priv->sorted_pictures = g_slist_remove (priv->sorted_pictures, picture);
               g_object_unref (picture);
 
-              break;
+              found = TRUE;
             }
+          g_object_unref (picture_from_ui);
         }
-      while (gtk_tree_model_iter_next (tree_model, &iter));
+      while (!found && gtk_tree_model_iter_next (tree_model, &iter));
     }
 
   /* Update upload size in state description */
@@ -2019,11 +2162,6 @@ _update_sensitiveness (FrogrMainView *self)
       for (i = 0; action_names[i]; i++)
         _update_sensitiveness_for_action (self, action_names[i], FALSE);
       g_strfreev (action_names);
-
-      /* Toolbar and contextual menu */
-      gtk_action_group_set_sensitive (priv->file_gtkactions, FALSE);
-      gtk_action_group_set_sensitive (priv->pictures_gtkactions, FALSE);
-      gtk_action_group_set_sensitive (priv->selection_gtkactions, FALSE);
       break;
 
     case FROGR_STATE_IDLE:
@@ -2045,11 +2183,6 @@ _update_sensitiveness (FrogrMainView *self)
       /* Elements from the GMenu - for available pictures */
       for (i = 0; i < G_N_ELEMENTS (iconview_actions); i++)
         _update_sensitiveness_for_action (self, iconview_actions[i], TRUE);
-
-      /* Toolbar and contextual menu */
-      gtk_action_group_set_sensitive (priv->file_gtkactions, TRUE);
-      gtk_action_group_set_sensitive (priv->pictures_gtkactions, has_pics);
-      gtk_action_group_set_sensitive (priv->selection_gtkactions, n_selected_pics);
       break;
 
     default:
@@ -2111,6 +2244,7 @@ _frogr_main_view_dispose (GObject *object)
 
   if (priv->tree_model)
     {
+      gtk_list_store_clear (GTK_LIST_STORE (priv->tree_model));
       g_object_unref (priv->tree_model);
       priv->tree_model = NULL;
     }
@@ -2133,6 +2267,7 @@ _frogr_main_view_finalize (GObject *object)
   g_free (priv->project_dir);
   g_free (priv->project_filepath);
   g_free (priv->state_description);
+  gtk_tree_path_free (priv->reference_picture);
 
   G_OBJECT_CLASS(frogr_main_view_parent_class)->finalize (object);
 }
