@@ -1,8 +1,8 @@
 /*
  * frogr-photoset.c -- An set in frogr (a photoset from flickr)
  *
- * Copyright (C) 2010-2011 Mario Sanchez Prada
- * Authors: Mario Sanchez Prada <msanchez@igalia.com>
+ * Copyright (C) 2010-2012 Mario Sanchez Prada
+ * Authors: Mario Sanchez Prada <msanchez@gnome.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 3 of the GNU General Public
@@ -16,10 +16,15 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, see <http://www.gnu.org/licenses/>
  *
+ * Parts of this file based on code from GConf, licensed as GPL
+ * version 2 or later (Copyright (C) 1999, 2000 Red Hat Inc)
  */
 
 #include "frogr-photoset.h"
 
+#include <sys/types.h>
+#include <unistd.h>
+
 #define FROGR_PHOTOSET_GET_PRIVATE(object)              \
   (G_TYPE_INSTANCE_GET_PRIVATE ((object),               \
                                 FROGR_TYPE_PHOTOSET,    \
@@ -34,6 +39,7 @@ struct _FrogrPhotoSetPrivate
   gchar *title;
   gchar *description;
   gchar *id;
+  gchar *local_id; /* For locally created sets only */
   gchar *primary_photo_id;
   gint n_photos;
 };
@@ -44,15 +50,73 @@ enum  {
   PROP_TITLE,
   PROP_DESCRIPTION,
   PROP_ID,
+  PROP_LOCAL_ID,
   PROP_PRIMARY_PHOTO_ID,
   PROP_N_PHOTOS
 };
 
 /* Prototypes */
 
+static gchar *_create_temporary_id_for_photoset (void);
+
 
 /* Private API */
 
+/* The following function is based in gconf_unique_key(), licensed as
+   GPL version 2 or later (Copyright (C) 1999, 2000 Red Hat Inc) */
+static gchar *
+_create_temporary_id_for_photoset (void)
+{
+  /* This function is hardly cryptographically random but should be
+     "good enough" */
+  static guint serial = 0;
+  gchar* key;
+  guint t, ut, p, u;
+  guint32 r;
+  GTimeVal tv;
+
+  g_get_current_time(&tv);
+
+  t = tv.tv_sec;
+  ut = tv.tv_usec;
+
+  p = getpid();
+  u = getuid();
+
+  /* don't bother to seed; if it's based on the time or any other
+     changing info we can get, we may as well just use that changing
+     info. since we don't seed we'll at least get a different number
+     on every call to this function in the same executable. */
+  r = g_random_int();
+
+  /* The letters may increase uniqueness by preventing "melds"
+     i.e. 01t01k01 and 0101t0k1 are not the same */
+  key = g_strdup_printf("%ut%uut%uu%up%ur%uk%u",
+                        /* Duplicate keys must be generated
+                           by two different program instances */
+                        serial,
+                        /* Duplicate keys must be generated
+                           in the same microsecond */
+                        t,
+                        ut,
+                        /* Duplicate keys must be generated by
+                           the same user */
+                        u,
+                        /* Duplicate keys must be generated by
+                           two programs that got the same PID */
+                        p,
+                        /* Duplicate keys must be generated with the
+                           same random seed and the same index into
+                           the series of pseudorandom values */
+                        r,
+                        /* Duplicate keys must result from running
+                           this function at the same stack location */
+                        GPOINTER_TO_UINT(&key));
+  ++serial;
+
+  return key;
+}
+
 static void
 _frogr_photoset_set_property (GObject *object,
                               guint prop_id,
@@ -72,6 +136,9 @@ _frogr_photoset_set_property (GObject *object,
     case PROP_ID:
       frogr_photoset_set_id (self, g_value_get_string (value));
       break;
+    case PROP_LOCAL_ID:
+      frogr_photoset_set_local_id (self, g_value_get_string (value));
+      break;
     case PROP_PRIMARY_PHOTO_ID:
       frogr_photoset_set_primary_photo_id (self, g_value_get_string (value));
       break;
@@ -90,24 +157,27 @@ _frogr_photoset_get_property (GObject *object,
                               GValue *value,
                               GParamSpec *pspec)
 {
-  FrogrPhotoSetPrivate *priv = FROGR_PHOTOSET_GET_PRIVATE (object);
+  FrogrPhotoSet *self = FROGR_PHOTOSET (object);
 
   switch (prop_id)
     {
     case PROP_TITLE:
-      g_value_set_string (value, priv->title);
+      g_value_set_string (value, frogr_photoset_get_title (self));
       break;
     case PROP_DESCRIPTION:
-      g_value_set_string (value, priv->description);
+      g_value_set_string (value, frogr_photoset_get_description (self));
       break;
     case PROP_ID:
-      g_value_set_string (value, priv->id);
+      g_value_set_string (value, frogr_photoset_get_id (self));
+      break;
+    case PROP_LOCAL_ID:
+      g_value_set_string (value, frogr_photoset_get_local_id (self));
       break;
     case PROP_PRIMARY_PHOTO_ID:
-      g_value_set_string (value, priv->primary_photo_id);
+      g_value_set_string (value, frogr_photoset_get_primary_photo_id (self));
       break;
     case PROP_N_PHOTOS:
-      g_value_set_int (value, priv->n_photos);
+      g_value_set_int (value, frogr_photoset_get_n_photos (self));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -124,6 +194,7 @@ _frogr_photoset_finalize (GObject* object)
   g_free (priv->title);
   g_free (priv->description);
   g_free (priv->id);
+  g_free (priv->local_id);
   g_free (priv->primary_photo_id);
 
   /* call super class */
@@ -162,7 +233,14 @@ frogr_photoset_class_init(FrogrPhotoSetClass *klass)
                                                         "Photoset ID from flickr",
                                                         NULL,
                                                         G_PARAM_READWRITE));
-
+  g_object_class_install_property (obj_class,
+                                   PROP_LOCAL_ID,
+                                   g_param_spec_string ("local-id",
+                                                        "local-id",
+                                                        "Photoset ID locally generated"
+                                                        "(for new sets only)",
+                                                        NULL,
+                                                        G_PARAM_READWRITE));
   g_object_class_install_property (obj_class,
                                    PROP_PRIMARY_PHOTO_ID,
                                    g_param_spec_string ("primary-photo-id",
@@ -194,6 +272,7 @@ frogr_photoset_init (FrogrPhotoSet *self)
   priv->title = NULL;
   priv->description = NULL;
   priv->id = NULL;
+  priv->local_id = NULL;
   priv->primary_photo_id = NULL;
   priv->n_photos = 0;
 }
@@ -202,18 +281,39 @@ frogr_photoset_init (FrogrPhotoSet *self)
 /* Public API */
 
 FrogrPhotoSet *
-frogr_photoset_new (const gchar *title,
+frogr_photoset_new (const gchar *id,
+                    const gchar *title,
                     const gchar *description)
 {
   g_return_val_if_fail (title, NULL);
   g_return_val_if_fail (description, NULL);
 
   return FROGR_PHOTOSET (g_object_new(FROGR_TYPE_PHOTOSET,
+                                      "id", id,
                                       "title", title,
                                       "description", description,
                                       NULL));
 }
 
+FrogrPhotoSet *
+frogr_photoset_new_local (const gchar *title,
+                          const gchar *description)
+{
+  FrogrPhotoSet *new_set;
+  gchar *id;
+
+  g_return_val_if_fail (title, NULL);
+  g_return_val_if_fail (description, NULL);
+
+  new_set = frogr_photoset_new (NULL, title, description);
+
+  /* We always need to have an id in locally created photosets */
+  id = _create_temporary_id_for_photoset ();
+  frogr_photoset_set_local_id (new_set, id);
+  g_free (id);
+
+  return new_set;
+}
 
 /* Data Managing functions */
 
@@ -290,6 +390,30 @@ frogr_photoset_set_id (FrogrPhotoSet *self,
   priv->id = g_strdup (id);
 }
 
+const gchar *
+frogr_photoset_get_local_id (FrogrPhotoSet *self)
+{
+  FrogrPhotoSetPrivate *priv = NULL;
+
+  g_return_val_if_fail(FROGR_IS_PHOTOSET(self), NULL);
+
+  priv = FROGR_PHOTOSET_GET_PRIVATE (self);
+  return (const gchar *)priv->local_id;
+}
+
+void
+frogr_photoset_set_local_id (FrogrPhotoSet *self,
+                             const gchar *id)
+{
+  FrogrPhotoSetPrivate *priv = NULL;
+
+  g_return_if_fail(FROGR_IS_PHOTOSET(self));
+
+  priv = FROGR_PHOTOSET_GET_PRIVATE (self);
+  g_free (priv->local_id);
+  priv->local_id = g_strdup (id);
+}
+
 const gchar *
 frogr_photoset_get_primary_photo_id (FrogrPhotoSet *self)
 {
@@ -337,3 +461,13 @@ frogr_photoset_set_n_photos (FrogrPhotoSet *self,
   priv->n_photos = n;
 }
 
+gboolean
+frogr_photoset_is_local (FrogrPhotoSet *self)
+{
+  FrogrPhotoSetPrivate *priv = NULL;
+
+  g_return_val_if_fail(FROGR_IS_PHOTOSET(self), FALSE);
+
+  priv = FROGR_PHOTOSET_GET_PRIVATE (self);
+  return priv->id == NULL && priv->local_id != NULL;
+}
