@@ -1,8 +1,8 @@
 /*
  * frogr-photoset.h -- An set in frogr (a photoset from flickr)
  *
- * Copyright (C) 2010-2011 Mario Sanchez Prada
- * Authors: Mario Sanchez Prada <msanchez@igalia.com>
+ * Copyright (C) 2010-2012 Mario Sanchez Prada
+ * Authors: Mario Sanchez Prada <msanchez@gnome.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 3 of the GNU General Public
@@ -50,15 +50,23 @@ struct _FrogrPhotoSetClass
 GType frogr_photoset_get_type(void) G_GNUC_CONST;
 
 /* Constructors */
-FrogrPhotoSet *frogr_photoset_new (const gchar *title,
+
+FrogrPhotoSet *frogr_photoset_new (const gchar *id,
+                                   const gchar *title,
                                    const gchar *description);
 
+FrogrPhotoSet *frogr_photoset_new_local (const gchar *title,
+                                         const gchar *description);
 /* Data managing methods */
 
 const gchar *frogr_photoset_get_id (FrogrPhotoSet *self);
 void frogr_photoset_set_id (FrogrPhotoSet *self,
                             const gchar *id);
 
+const gchar *frogr_photoset_get_local_id (FrogrPhotoSet *self);
+void frogr_photoset_set_local_id (FrogrPhotoSet *self,
+                                  const gchar *id);
+
 const gchar *frogr_photoset_get_title (FrogrPhotoSet *self);
 void frogr_photoset_set_title (FrogrPhotoSet *self,
                                const gchar *title);
@@ -74,6 +82,9 @@ void frogr_photoset_set_primary_photo_id (FrogrPhotoSet *self,
 gint frogr_photoset_get_n_photos (FrogrPhotoSet *self);
 void frogr_photoset_set_n_photos (FrogrPhotoSet *self,
                                   gint n);
+
+gboolean frogr_photoset_is_local (FrogrPhotoSet *self);
+
 G_END_DECLS
 
 #endif
