@@ -2,7 +2,7 @@
  * frogr-picture.c -- A picture in frogr
  *
  * Copyright (C) 2009-2012 Mario Sanchez Prada
- * Authors: Mario Sanchez Prada <msanchez@igalia.com>
+ * Authors: Mario Sanchez Prada <msanchez@gnome.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 3 of the GNU General Public
@@ -20,6 +20,11 @@
 
 #include "frogr-picture.h"
 
+#include "frogr-controller.h"
+#include "frogr-model.h"
+
+#include <json-glib/json-glib.h>
+
 #define TAGS_DELIMITER " "
 
 #define FROGR_PICTURE_GET_PRIVATE(object)               \
@@ -27,7 +32,11 @@
                                 FROGR_TYPE_PICTURE,     \
                                 FrogrPicturePrivate))
 
-G_DEFINE_TYPE (FrogrPicture, frogr_picture, G_TYPE_OBJECT)
+static void json_serializable_init (JsonSerializableIface *iface);
+
+G_DEFINE_TYPE_EXTENDED (FrogrPicture, frogr_picture, G_TYPE_OBJECT, 0,
+                        G_IMPLEMENT_INTERFACE (JSON_TYPE_SERIALIZABLE,
+                                               json_serializable_init))
 
 /* Private struct */
 typedef struct _FrogrPicturePrivate FrogrPicturePrivate;
@@ -51,7 +60,9 @@ struct _FrogrPicturePrivate
   gboolean show_in_search;
   gboolean send_location;
 
-  gulong filesize; /* In KB */
+  gboolean is_video;
+
+  guint filesize; /* In KB */
   gchar *datetime; /* ASCII, locale dependent, string */
 
   GSList *photosets;
@@ -77,11 +88,11 @@ enum  {
   PROP_LOCATION,
   PROP_SHOW_IN_SEARCH,
   PROP_SEND_LOCATION,
+  PROP_IS_VIDEO,
   PROP_FILESIZE,
   PROP_DATETIME,
   PROP_PHOTOSETS,
-  PROP_GROUPS,
-  PROP_PIXBUF
+  PROP_GROUPS
 };
 
 /* Prototypes */
@@ -91,6 +102,21 @@ static void _add_tags_to_tags_list (FrogrPicture *self,
                                     const gchar *tags_string);
 static void _update_tags_string (FrogrPicture *self);
 
+static gint _compare_photosets (FrogrPhotoSet *photoset1, FrogrPhotoSet *photoset2);
+static gint _compare_groups (FrogrGroup *group1, FrogrGroup *group2);
+
+static JsonNode *_serialize_list (GSList *objects_list, GType g_type);
+static gboolean _deserialize_list (JsonNode *node, GValue *value, GType g_type);
+
+static JsonNode *_serialize_property (JsonSerializable *serializable,
+                                      const gchar *name,
+                                      const GValue *value,
+                                      GParamSpec *pspec);
+static gboolean _deserialize_property (JsonSerializable *serializable,
+                                       const gchar *name,
+                                       GValue *value,
+                                       GParamSpec *pspec,
+                                       JsonNode *node);
 /* Private API */
 
 static gboolean
@@ -184,6 +210,198 @@ _update_tags_string (FrogrPicture *self)
     }
 }
 
+static gint
+_compare_photosets (FrogrPhotoSet *photoset1, FrogrPhotoSet *photoset2)
+{
+  const gchar *id1 = NULL;
+  const gchar *id2 = NULL;
+  const gchar *local_id1 = NULL;
+  const gchar *local_id2 = NULL;
+
+  g_return_val_if_fail (FROGR_IS_PHOTOSET (photoset1), 1);
+  g_return_val_if_fail (FROGR_IS_PHOTOSET (photoset2), -1);
+
+  if (photoset1 == photoset2)
+    return 0;
+
+  id1 = frogr_photoset_get_id (photoset1);
+  id2 = frogr_photoset_get_id (photoset2);
+  if (id1 != NULL && id2 != NULL)
+    return g_strcmp0 (id1, id2);
+
+  local_id1 = frogr_photoset_get_local_id (photoset1);
+  local_id2 = frogr_photoset_get_local_id (photoset2);
+  if (local_id1 != NULL && local_id2 != NULL)
+    return g_strcmp0 (local_id1, local_id2);
+
+  if (id1 != NULL)
+    return 1;
+  else
+    return -1;
+}
+
+static gint
+_compare_groups (FrogrGroup *group1, FrogrGroup *group2)
+{
+  const gchar *id1 = NULL;
+  const gchar *id2 = NULL;
+
+  g_return_val_if_fail (FROGR_IS_GROUP (group1), 1);
+  g_return_val_if_fail (FROGR_IS_GROUP (group2), -1);
+
+  if (group1 == group2)
+    return 0;
+
+  id1 = frogr_group_get_id (group1);
+  id2 = frogr_group_get_id (group2);
+  if (id1 != NULL && id2 != NULL)
+    return g_strcmp0 (id1, id2);
+
+  if (id1 != NULL)
+    return 1;
+  else
+    return -1;
+}
+
+static JsonNode *
+_serialize_list (GSList *objects_list, GType g_type)
+{
+  JsonArray *json_array = NULL;
+  JsonNode *list_node = NULL;
+  GObject *object = NULL;
+  GSList *item = NULL;
+  const gchar *id = NULL;
+
+  json_array = json_array_new ();
+  for (item = objects_list; item; item = g_slist_next (item))
+    {
+      /* We just serialize the ID of the group / set */
+      object = G_OBJECT (item->data);
+      if (g_type == FROGR_TYPE_PHOTOSET)
+        {
+          id = frogr_photoset_get_id (FROGR_PHOTOSET (object));
+          if (!id)
+            id = frogr_photoset_get_local_id (FROGR_PHOTOSET (object));
+        }
+      else if (g_type == FROGR_TYPE_GROUP)
+        id = frogr_group_get_id (FROGR_GROUP (object));
+
+      if (id)
+        json_array_add_string_element (json_array, id);
+    }
+
+  list_node = json_node_new(JSON_NODE_ARRAY);
+  json_node_set_array (list_node, json_array);
+
+  return list_node;
+}
+
+static gboolean
+_deserialize_list (JsonNode *node, GValue *value, GType g_type)
+{
+  FrogrModel *model = NULL;
+  JsonArray *array = NULL;
+  GSList *objects = NULL;
+  GObject *object = NULL;
+  guint n_elements = 0;
+  guint i = 0;
+
+  g_return_val_if_fail (node != NULL, FALSE);
+  g_return_val_if_fail (JSON_NODE_HOLDS_ARRAY (node), FALSE);
+
+  array = json_node_get_array (node);
+  n_elements = json_array_get_length (array);
+  if (!n_elements)
+    return TRUE;
+
+  /* We need to get the groups and sets from the model by ID, so it's
+     mandatory to have imported those first for this to work OK */
+  model = frogr_controller_get_model (frogr_controller_get_instance ());
+  for (i = 0; i < n_elements; i++)
+    {
+      const gchar *id = NULL;
+      id = json_array_get_string_element (array, i);
+      if (g_type == FROGR_TYPE_PHOTOSET)
+        object = G_OBJECT (frogr_model_get_photoset_by_id (model, id));
+      if (g_type == FROGR_TYPE_GROUP)
+        object = G_OBJECT (frogr_model_get_group_by_id (model, id));
+
+      if (object)
+        objects = g_slist_prepend (objects, g_object_ref (object));
+    }
+  g_value_set_pointer (value, g_slist_reverse (objects));
+
+  return TRUE;
+}
+
+static JsonNode *
+_serialize_property (JsonSerializable *serializable,
+                     const gchar *name,
+                     const GValue *value,
+                     GParamSpec *pspec)
+{
+  FrogrPicturePrivate *priv = NULL;
+  JsonNode *json_node = NULL;
+
+  g_return_val_if_fail (FROGR_IS_PICTURE (serializable), FALSE);
+
+  priv = FROGR_PICTURE_GET_PRIVATE (serializable);
+  if (g_str_equal (name, "photosets"))
+    json_node = _serialize_list (priv->photosets, FROGR_TYPE_PHOTOSET);
+  else if (g_str_equal (name, "groups"))
+    json_node = _serialize_list (priv->groups, FROGR_TYPE_GROUP);
+  else
+    {
+      /* Default serialization here */
+      JsonSerializableIface *iface = NULL;
+      iface = g_type_default_interface_peek (JSON_TYPE_SERIALIZABLE);
+      json_node = iface->serialize_property (serializable,
+                                             name,
+                                             value,
+                                             pspec);
+    }
+
+  return json_node; /* NULL indicates default deserializer */
+}
+
+static gboolean
+_deserialize_property (JsonSerializable *serializable,
+                       const gchar *name,
+                       GValue *value,
+                       GParamSpec *pspec,
+                       JsonNode *node)
+{
+  gboolean result = FALSE;
+
+  g_return_val_if_fail (node != NULL, FALSE);
+
+  if (g_str_equal (name, "photosets"))
+    result = _deserialize_list (node, value, FROGR_TYPE_PHOTOSET);
+  else if (g_str_equal (name, "groups"))
+    result = _deserialize_list (node, value, FROGR_TYPE_GROUP);
+  else
+    {
+      /* Default deserialization */
+      JsonSerializableIface *iface = NULL;
+
+      iface = g_type_default_interface_peek (JSON_TYPE_SERIALIZABLE);
+      result = iface->deserialize_property (serializable,
+                                            name,
+                                            value,
+                                            pspec,
+                                            node);
+    }
+
+  return result;
+}
+
+static void
+json_serializable_init (JsonSerializableIface *iface)
+{
+  iface->serialize_property = _serialize_property;
+  iface->deserialize_property = _deserialize_property;
+}
+
 static void
 _frogr_picture_set_property (GObject *object,
                              guint prop_id,
@@ -191,6 +409,7 @@ _frogr_picture_set_property (GObject *object,
                              GParamSpec *pspec)
 {
   FrogrPicture *self = FROGR_PICTURE (object);
+  FrogrPicturePrivate *priv = FROGR_PICTURE_GET_PRIVATE (self);
 
   switch (prop_id)
     {
@@ -198,7 +417,7 @@ _frogr_picture_set_property (GObject *object,
       frogr_picture_set_id (self, g_value_get_string (value));
       break;
     case PROP_FILEURI:
-      frogr_picture_set_fileuri (self, g_value_get_string (value));
+      priv->fileuri = g_value_dup_string (value);
       break;
     case PROP_TITLE:
       frogr_picture_set_title (self, g_value_get_string (value));
@@ -236,8 +455,11 @@ _frogr_picture_set_property (GObject *object,
     case PROP_SEND_LOCATION:
       frogr_picture_set_send_location (self, g_value_get_boolean (value));
       break;
+    case PROP_IS_VIDEO:
+      priv->is_video = g_value_get_boolean (value);
+      break;
     case PROP_FILESIZE:
-      frogr_picture_set_filesize (self, g_value_get_long (value));
+      frogr_picture_set_filesize (self, g_value_get_uint (value));
       break;
     case PROP_DATETIME:
       frogr_picture_set_datetime (self, g_value_get_string (value));
@@ -248,9 +470,6 @@ _frogr_picture_set_property (GObject *object,
     case PROP_GROUPS:
       frogr_picture_set_groups (self, (GSList *) g_value_get_pointer (value));
       break;
-    case PROP_PIXBUF:
-      frogr_picture_set_pixbuf (self, GDK_PIXBUF (g_value_get_object (value)));
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -309,8 +528,11 @@ _frogr_picture_get_property (GObject *object,
     case PROP_SEND_LOCATION:
       g_value_set_boolean (value, priv->send_location);
       break;
+    case PROP_IS_VIDEO:
+      g_value_set_boolean (value, priv->is_video);
+      break;
     case PROP_FILESIZE:
-      g_value_set_long (value, priv->filesize);
+      g_value_set_uint (value, priv->filesize);
       break;
     case PROP_DATETIME:
       g_value_set_string (value, priv->datetime);
@@ -321,9 +543,6 @@ _frogr_picture_get_property (GObject *object,
     case PROP_GROUPS:
       g_value_set_pointer (value, priv->groups);
       break;
-    case PROP_PIXBUF:
-      g_value_set_object (value, priv->pixbuf);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -335,17 +554,8 @@ _frogr_picture_dispose (GObject* object)
 {
   FrogrPicturePrivate *priv = FROGR_PICTURE_GET_PRIVATE (object);
 
-  if (priv->pixbuf)
-    {
-      g_object_unref (priv->pixbuf);
-      priv->pixbuf = NULL;
-    }
-
-  if (priv->location)
-    {
-      g_object_unref (priv->location);
-      priv->location = NULL;
-    }
+  g_clear_object (&priv->pixbuf);
+  g_clear_object (&priv->location);
 
   if (priv->photosets)
     {
@@ -413,7 +623,7 @@ frogr_picture_class_init(FrogrPictureClass *klass)
                                                         "Full fileuri at disk "
                                                         "for the picture",
                                                         NULL,
-                                                        G_PARAM_READWRITE));
+                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
   g_object_class_install_property (obj_class,
                                    PROP_TITLE,
                                    g_param_spec_string ("title",
@@ -498,7 +708,6 @@ frogr_picture_class_init(FrogrPictureClass *klass)
                                                         "Location for this picture",
                                                         FROGR_TYPE_LOCATION,
                                                         G_PARAM_READWRITE));
-
   g_object_class_install_property (obj_class,
                                    PROP_SHOW_IN_SEARCH,
                                    g_param_spec_boolean ("show-in-search",
@@ -518,13 +727,20 @@ frogr_picture_class_init(FrogrPictureClass *klass)
                                                          G_PARAM_READWRITE));
   g_object_class_install_property (obj_class,
                                    PROP_FILESIZE,
-                                   g_param_spec_long ("filesize",
+                                   g_param_spec_uint ("filesize",
                                                       "filesize",
                                                       "Filesize in KB for the file",
-                                                      G_MINLONG,
-                                                      G_MAXLONG,
+                                                      0,
+                                                      G_MAXUINT,
                                                       0,
                                                       G_PARAM_READWRITE));
+  g_object_class_install_property (obj_class,
+                                   PROP_IS_VIDEO,
+                                   g_param_spec_boolean ("is-video",
+                                                         "is-video",
+                                                         "Whether FrogrPicture represents a video",
+                                                         FALSE,
+                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
   g_object_class_install_property (obj_class,
                                    PROP_DATETIME,
                                    g_param_spec_string ("datetime",
@@ -544,14 +760,6 @@ frogr_picture_class_init(FrogrPictureClass *klass)
                                                          "groups",
                                                          "List of groups the picture is in",
                                                          G_PARAM_READWRITE));
-  g_object_class_install_property (obj_class,
-                                   PROP_PIXBUF,
-                                   g_param_spec_object ("pixbuf",
-                                                        "pixbuf",
-                                                        "Pre-loaded GdkPixbuf "
-                                                        "for this picture",
-                                                        GDK_TYPE_PIXBUF,
-                                                        G_PARAM_READWRITE));
 
   g_type_class_add_private (obj_class, sizeof (FrogrPicturePrivate));
 }
@@ -581,6 +789,8 @@ frogr_picture_init (FrogrPicture *self)
   priv->show_in_search = TRUE;
   priv->send_location = FALSE;
 
+  priv->is_video = FALSE;
+
   priv->filesize = 0;
   priv->datetime = NULL;
 
@@ -598,7 +808,8 @@ frogr_picture_new (const gchar *fileuri,
                    const gchar *title,
                    gboolean public,
                    gboolean family,
-                   gboolean friend)
+                   gboolean friend,
+                   gboolean is_video)
 {
   g_return_val_if_fail (fileuri, NULL);
   g_return_val_if_fail (title, NULL);
@@ -609,6 +820,7 @@ frogr_picture_new (const gchar *fileuri,
                                      "is-public", public,
                                      "is-family", family,
                                      "is-friend", friend,
+                                     "is-video", is_video,
                                      NULL));
 }
 
@@ -650,19 +862,6 @@ frogr_picture_get_fileuri (FrogrPicture *self)
   return (const gchar *)priv->fileuri;
 }
 
-void
-frogr_picture_set_fileuri (FrogrPicture *self,
-                           const gchar *fileuri)
-{
-  FrogrPicturePrivate *priv = NULL;
-
-  g_return_if_fail(FROGR_IS_PICTURE(self));
-
-  priv = FROGR_PICTURE_GET_PRIVATE (self);
-  g_free (priv->fileuri);
-  priv->fileuri = g_strdup (fileuri);
-}
-
 const gchar *
 frogr_picture_get_title (FrogrPicture *self)
 {
@@ -1001,7 +1200,17 @@ frogr_picture_set_pixbuf (FrogrPicture *self,
   priv->pixbuf = GDK_IS_PIXBUF (pixbuf) ? g_object_ref (pixbuf) : NULL;
 }
 
-gulong frogr_picture_get_filesize (FrogrPicture *self)
+gboolean frogr_picture_is_video (FrogrPicture *self)
+{
+  FrogrPicturePrivate *priv = NULL;
+
+  g_return_val_if_fail(FROGR_IS_PICTURE(self), FALSE);
+
+  priv = FROGR_PICTURE_GET_PRIVATE (self);
+  return priv->is_video;
+}
+
+guint frogr_picture_get_filesize (FrogrPicture *self)
 {
   FrogrPicturePrivate *priv = NULL;
 
@@ -1011,7 +1220,7 @@ gulong frogr_picture_get_filesize (FrogrPicture *self)
   return priv->filesize;
 }
 
-void frogr_picture_set_filesize (FrogrPicture *self, gulong filesize)
+void frogr_picture_set_filesize (FrogrPicture *self, guint filesize)
 {
   FrogrPicturePrivate *priv = NULL;
 
@@ -1068,12 +1277,7 @@ frogr_picture_set_photosets (FrogrPicture *self, GSList *photosets)
   g_slist_foreach (priv->photosets, (GFunc) g_object_unref, NULL);
   g_slist_free (priv->photosets);
 
-  priv->photosets = NULL;
-  if (photosets)
-    {
-      priv->photosets = g_slist_copy (photosets);
-      g_slist_foreach (priv->photosets, (GFunc) g_object_ref, NULL);
-    }
+  priv->photosets = photosets;
 }
 
 void
@@ -1108,7 +1312,8 @@ frogr_picture_in_photoset (FrogrPicture *self, FrogrPhotoSet *photoset)
   g_return_val_if_fail(FROGR_IS_PICTURE(self), FALSE);
 
   priv = FROGR_PICTURE_GET_PRIVATE (self);
-  if (g_slist_index (priv->photosets, photoset) != -1)
+  if (g_slist_find_custom (priv->photosets, photoset,
+                           (GCompareFunc)_compare_photosets))
     return TRUE;
 
   return FALSE;
@@ -1138,12 +1343,23 @@ frogr_picture_set_groups (FrogrPicture *self, GSList *groups)
   g_slist_foreach (priv->groups, (GFunc) g_object_unref, NULL);
   g_slist_free (priv->groups);
 
-  priv->groups = NULL;
-  if (groups)
-    {
-      priv->groups = g_slist_copy (groups);
-      g_slist_foreach (priv->groups, (GFunc) g_object_ref, NULL);
-    }
+  priv->groups = groups;
+}
+
+void
+frogr_picture_add_group (FrogrPicture *self, FrogrGroup *group)
+{
+  FrogrPicturePrivate *priv = NULL;
+
+  g_return_if_fail(FROGR_IS_PICTURE(self));
+  g_return_if_fail(FROGR_IS_GROUP(group));
+
+  /* Do not add the same set twice */
+  if (frogr_picture_in_group (self, group))
+    return;
+
+  priv = FROGR_PICTURE_GET_PRIVATE (self);
+  priv->groups = g_slist_append (priv->groups, g_object_ref (group));
 }
 
 void
@@ -1162,7 +1378,8 @@ frogr_picture_in_group (FrogrPicture *self, FrogrGroup *group)
   g_return_val_if_fail(FROGR_IS_PICTURE(self), FALSE);
 
   priv = FROGR_PICTURE_GET_PRIVATE (self);
-  if (g_slist_index (priv->groups, group) != -1)
+  if (g_slist_find_custom (priv->groups, group,
+                           (GCompareFunc)_compare_groups))
     return TRUE;
 
   return FALSE;
