@@ -1,7 +1,7 @@
 /*
  * frogr-settings-dialog.c -- Main settings dialog
  *
- * Copyright (C) 2009-2012 Mario Sanchez Prada
+ * Copyright (C) 2009-2014 Mario Sanchez Prada
  * Authors: Mario Sanchez Prada <msanchez@gnome.org>
  *
  * This program is free software; you can redistribute it and/or
@@ -169,7 +169,8 @@ _add_general_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
   GtkWidget *vbox = NULL;
   GtkWidget *box1 = NULL;
   GtkWidget *box2 = NULL;
-  GtkWidget *padding_hbox = NULL;
+  GtkWidget *box3 = NULL;
+  GtkWidget *box4 = NULL;
   GtkWidget *align = NULL;
   GtkWidget *label = NULL;
   GtkWidget *combo = NULL;
@@ -196,29 +197,35 @@ _add_general_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
 
   box1 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
   gtk_box_set_homogeneous (GTK_BOX (box1), FALSE);
-  box2 = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
+  box2 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
   gtk_box_set_homogeneous (GTK_BOX (box2), FALSE);
 
-  _add_toggleable_item (self, GTK_BOX (box2), NULL, TRUE, _("_Private"), &priv->private_rb);
-  _add_toggleable_item (self, GTK_BOX (box2), GTK_RADIO_BUTTON (priv->private_rb),
+  box3 = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+  gtk_box_set_homogeneous (GTK_BOX (box3), FALSE);
+
+  _add_toggleable_item (self, GTK_BOX (box3), NULL,
+                        TRUE, _("_Private"), &priv->private_rb);
+
+  _add_toggleable_item (self, GTK_BOX (box3), GTK_RADIO_BUTTON (priv->private_rb),
                         FALSE, _("P_ublic"), &priv->public_rb);
 
+  gtk_box_pack_start (GTK_BOX (box2), box3, FALSE, FALSE, 0);
+
+  box3 = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+  gtk_box_set_homogeneous (GTK_BOX (box3), FALSE);
+
+  box4 = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+  gtk_box_set_homogeneous (GTK_BOX (box4), FALSE);
+
+  _add_toggleable_item (self, GTK_BOX (box4), NULL, FALSE,
+                        _("_Family"), &priv->family_cb);
+  _add_toggleable_item (self, GTK_BOX (box4), NULL, FALSE,
+                        _("F_riends"), &priv->friend_cb);
+
+  gtk_box_pack_start (GTK_BOX (box3), box4, FALSE, FALSE, 12);
+  gtk_box_pack_start (GTK_BOX (box2), box3, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (box1), box2, FALSE, FALSE, 0);
 
-  box2 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
-  gtk_box_set_homogeneous (GTK_BOX (box2), FALSE);
-
-  _add_toggleable_item (self, GTK_BOX (box2), NULL, FALSE,
-                        _("Visible to _Family"), &priv->family_cb);
-  _add_toggleable_item (self, GTK_BOX (box2), NULL, FALSE,
-                        _("Visible to F_riends"), &priv->friend_cb);
-
-  padding_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-  gtk_box_set_homogeneous (GTK_BOX (padding_hbox), FALSE);
-
-  gtk_box_pack_start (GTK_BOX (padding_hbox), box2, FALSE, FALSE, 12);
-  gtk_box_pack_start (GTK_BOX (box1), padding_hbox, FALSE, FALSE, 0);
-
   gtk_box_pack_start (GTK_BOX (vbox), box1, FALSE, FALSE, 0);
 
   /* Default Content type */
@@ -992,9 +999,9 @@ frogr_settings_dialog_init (FrogrSettingsDialog *self)
 
   /* Create widgets */
   gtk_dialog_add_buttons (GTK_DIALOG (self),
-                          GTK_STOCK_CANCEL,
+                          _("_Cancel"),
                           GTK_RESPONSE_CANCEL,
-                          GTK_STOCK_CLOSE,
+                          _("_Close"),
                           GTK_RESPONSE_CLOSE,
                           NULL);
   gtk_container_set_border_width (GTK_CONTAINER (self), 6);
